#include <signal.h>
#include <sys/wait.h>

class chld {
  static const int MAXSTR = 1024;
  int fi[ 2 ], fo[ 2 ];			// pipe – для ввода и для вывода
  pid_t pid;
  char** create( const char *s );
public:
  char buf[ MAXSTR ];
  chld( const char*, int* fdi = NULL, int* fdo = NULL );
  ~chld( void ); 
  friend chld& operator <<( chld& c, const char *s );
  friend chld& operator >>( chld& c, char *s );
};
// это внутренняя private функция-член разбора ком. строки 
// и создания списка параметров запуска 
char** chld::create( const char *s ) {
  char *p = (char*)s, *f;
  int n;
  for( n = 0; ; n++ ) {
    if( ( p = strpbrk( p, " " ) ) == NULL ) break;
    while( *p == ' ' ) p++;      
  }; 
  char **pv = new char*[ n + 2 ];
  for( int i = 0; i < n + 2; i++ ) pv[ i ] = NULL;
  p = (char*)s;
  f = strpbrk( p, " " );   
  for( n = 0; ; n++ ) {
    int k = ( f - p );
    pv[ n ] = new char[ k + 1 ];
    strncpy( pv[ n ], p, k );
    pv[ n ][ k ] = '\0';
    p = f;
    while( *p == ' ' ) p++;      
    if( ( f = strpbrk( p, " " ) ) == NULL ) {
      pv[ n + 1 ] = strdup( p ); 
      break;
    }
  }
  return pv;
};
// вот главное «действо» класса – конструктор, здесь переназначаются
// потоки ввода вывода (SYSIN & SYSOUT), клонируется вызывающий
// процесс, и в нём вызывается новый процесс клиент со своими
// параметрами:
chld::chld( const char* pr,  int* fdi, int* fdo ) {
  if( pipe( fi ) || pipe( fo ) ) perror( "pipe" ), exit( EXIT_FAILURE );
  // здесь создаётся список параметров запуска (если нужно)
  char **pv = create( pr );
  pid = fork();
  switch( pid ) {
    case -1: perror( "fork" ), exit( EXIT_FAILURE );
    case  0:   // дочерний клон
      close( fi[ 1 ] ), close( fo[ 0 ] );
      if( dup2( fi[ 0 ], STDIN_FILENO ) == -1 ||
          dup2( fo[ 1 ], STDOUT_FILENO ) == -1 )
        perror( "dup2" ), exit( EXIT_FAILURE );
        close( fi[ 0 ] ), close( fo[ 1 ] );
        // запуск консольного клиента
        if( execvp( pv[ 0 ], pv ) == -1 )
          perror( "execl" ), exit( EXIT_FAILURE );
        break;
    default:   // родительский процесс
        for( int i = 0;; i++ )
          if( pv[ i ] != NULL ) delete pv[ i ]; else break;
        delete [] pv;
        close( fi[ 0 ] ), close( fo[ 1 ] );
        if( fdi != NULL ) *fdi = fo[ 0 ];
        if( fdo != NULL ) *fdo = fi[ 1 ];
        break;
   };
}
chld::~chld( void ) {
   if( kill( pid, SIGKILL ) != 0 )
      perror( "kill" ), exit( EXIT_FAILURE );
   if( -1 == waitpid( pid, NULL, WNOHANG ) )
      perror( "waitpid" ), exit( EXIT_FAILURE );
}
// оператор записи того, что дочерний процесс ожидает получить с SYSIN
chld& operator <<( chld& c, const char *s ) {
   if( strlen( s ) < sizeof( c.buf ) - 1 ) strcpy( c.buf, s );
   else perror( "write length" ), exit( EXIT_FAILURE );
   if( *( c.buf + strlen( c.buf ) - 1 ) != '\n' ) strcat( c.buf, "\n" ); 
   if( write( c.fi[ 1 ], c.buf, strlen( c.buf ) ) == -1 )
      perror( "write pipe" ), exit( EXIT_FAILURE );
   return c;
}
// оператор чтения того, что выводит в SYSOUT дочерний процесс
chld& operator >>( chld& c, char *s ) {
   int n = read( c.fo[ 0 ], c.buf, sizeof( c.buf ) - 1 );
   if( n == -1 ) perror( "read pipe" ), exit( EXIT_FAILURE );
   c.buf[ n ] = '\0';
   strcpy( s, c.buf );
   return c;
}
