#include  <syslog.h>
#include <sys/wait.h>

#include  "common.h"

int main( void ) {    // добавить запись UDP порта в /etc/services и в конфигурацию в /etc/xinetd.d
   char rbuf[ MAXLEN ];
   openlog( NULL, LOG_NDELAY, LOG_USER );
   syslog( LOG_NOTICE, "server [%d] start", getpid() );
   while( 1 ) {
      struct sockaddr_in adr;
      socklen_t len = sizeof( struct sockaddr_in );
      int n = recvfrom( STDIN_FILENO, rbuf, MAXLEN, 0,  // чтение пакета из SYSIN для xinetd:
                        (struct sockaddr*)&adr, &len ); 
      if( n >= 0 ) syslog( LOG_NOTICE, "server read %d byte", n );
      if( n <= 0 ) {
         if( n < 0 ) syslog( LOG_ERR, "recvfrom error: %m" ), exit( EXIT_FAILURE );
         break;
      }
      rbuf[ n ] = '\0';
      int sc = socket( AF_INET, SOCK_DGRAM, 0 );        // open a UDP socket
      if( sc < 0 ) syslog( LOG_ERR, "socket error: %m" ), exit( EXIT_FAILURE );
      if( connect( sc, (struct sockaddr*)&adr, sizeof( struct sockaddr_in ) ) < 0 )
         syslog( LOG_ERR, "connect error: %m" ), exit( EXIT_FAILURE );
      pid_t pid = fork();
      if( 0 == pid ) {                                  // обработка в дочернем процессе
         close( STDIN_FILENO );
         close( STDOUT_FILENO ); 
         sleep( 1 );                                    // имитация времени обработки
         char wbuf[ MAXLEN + 8 ];
         sprintf( wbuf, "[%d]%s%s", getpid(), SDELIM, rbuf );
         if( write( sc, wbuf, strlen( wbuf ) ) < 0 )    // ретрансляция данных
            syslog( LOG_ERR, "write error: %m" ), exit( EXIT_FAILURE );
         exit( EXIT_SUCCESS );                          // завершение обрабатывающего процесса
      }
      else if( pid > 0 ) { 
         if( n != 1 ) waitpid( -1, NULL, WNOHANG );     // продолжение работы
         else {                                         // пустая "\n" строка от клиента
            wait( NULL );
            exit( EXIT_SUCCESS );
         }
      }
      else syslog( LOG_ERR, "fork error: %m" ), exit( EXIT_FAILURE );
   }
   syslog( LOG_NOTICE, "server [%d] exit", getpid() );
   closelog();
   exit( EXIT_SUCCESS );
};
